# uc.pm
# =====
# Plugin Source

package Mail::SpamAssassin::Plugin::uc;
use Mail::SpamAssassin::Plugin;
use Mail::SpamAssassin::Util;
use Mail::SpamAssassin::Logger;
use Mail::SpamAssassin::PerMsgStatus;
use List::MoreUtils qw(uniq);
use Mail::SpamAssassin::PerMsgStatus;
use strict;
use bytes;
use vars qw(@ISA);
@ISA = qw(Mail::SpamAssassin::Plugin);
sub new {
  my $class = shift;
  my $mailsaobject = shift;

# some boilerplate...
  $class = ref($class) || $class;
  my $self = $class->SUPER::new($mailsaobject);
  bless ($self, $class);
  $self->set_config($mailsaobject->{conf});

  $self->register_eval_rule('call_uc');
  return $self;
}

# set_config
# ----------
# Load configuration parameters
sub set_config {
 #dbg#dbg("uc: set_config: Loading Configuration");
 my ($self, $conf) = @_; 
  my @cmds = ();
  push (@cmds, {
        setting => 'uc_sql_dsn',
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING,
  });
  push (@cmds, {
        setting => 'uc_sql_username',
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING,
  });
  push (@cmds, {
        setting => 'uc_sql_password',
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING,
  });
 $conf->{parser}->register_commands(\@cmds);
 #dbg#dbg("uc: set_config: Configuration Loaded");
}


# call_uc
# -------
# Main Proc called by uc.cf
sub call_uc {
 #dbg#dbg("uc: call_uc: Starting Url Check");
 # Get uri function call
 my ($self, $permsgstatus) = @_;
 my @uris = $permsgstatus->get_uri_list();
 my $uri = "";
 my @arr = ();
 # Loop into found uris
 foreach $uri (@uris) {
  # Skip unnecessary uri type
  next if ($uri =~ m/^(cid|mailto|javascript):/i); 
  #dbg#dbg("uc: call_uc: Found uri line: " . $uri);
  # Remove trailing
  $uri =~ s/http:\/\///gi;
  $uri =~ s/https:\/\///gi;
  # Remove after /
  $uri =~ s/\/.*//gi;
  # Remove before @ 
  $uri =~ s/.*\@//gi;
  #dbg#dbg("uc: call_uc: Cleaned uri: " . $uri);
  # Push uri in array
  push @arr, $uri;
 }
 # Make Uniques uri in arrray
 @arr = uniq @arr;
 # DSN
 my $sql = "";
 my $ref = "";
 my $rv=0;
 my $numfound=0;
 my $dsn = $self->{main}->{conf}->{uc_sql_dsn};
 #dbg#dbg("uc: call_uc: DSN: " . $dsn);
 require DBI;
 my $dbuser = $self->{main}->{conf}->{uc_sql_username};
 my $dbpass = $self->{main}->{conf}->{uc_sql_password};
 # Connect DB
 my $dbh = DBI->connect($dsn, $dbuser, $dbpass, {'PrintError' => 0});
 #dbg#dbg("uc: call_uc: Db User: " . $dbuser);
 #dbg#dbg("uc: call_uc: Db Password: " . $dbpass);
 # Scan array
 foreach my $elem (@arr) {
  # Remove some blank lines
  next if ($elem eq ''); 
  #dbg#dbg("uc: call_uc: Unique element: " . $elem);
  # Search unique element in database
  $sql =  "SELECT service FROM uc WHERE name = '$elem' ";
  $rv = $dbh->do($sql);
  if ($rv) {
   #dbg#dbg("uc: call_uc: Rv Status: " . $rv);
   if ($rv ge 1) {
    dbg("uc: call_uc: Found " . $elem);
    ++$numfound;
   }
  } else {
   die "uc: call_uc: Sql Error: " . $dbh->errstr;
  }
 }
 # Disconnect DB
 $dbh->disconnect();
 #dbg#dbg("uc: call_uc: NumFound: " . $numfound);
 # Positive return if numfound not 0
 if ($numfound != 0) {
  dbg("uc: call_uc: End Match Rule");
  return 1;
 } 
 # At default return 0 if none found
 dbg("uc: call_uc: End Not Found" );
 return 0;
}

1;

