# uc.load.sh
# ==========
# Load or refresh mailscanner -> uc table from various sources

# Export work folder and changing directory
export F="/etc/mail/spamassassin"
export FT="/tmp/uc"

# Put here your mysql root password
MYSQLROOT=<YOUR PASSWORD> 


# PhishTank ID
PTID=<YOURID>

# Hosts-File.net Extension list to load
HFLIST="emd exp fsa grm  hfs  hjk  pha  psh  wrz"

# Note : same hostnames have service field valued as latest service loaded




# START

# WorkDir Creation
if [[ ! -e $FT ]]; then
 mkdir $FT
 chmod 777 $FT
fi

# Get current date and time for sql use
dtd=`date +'%F'`
dtt=`date +'%T'`


# HOSTS-FILE.NET
# --------------
# This service have a lot of files divided in sections
# We load :
# emd - Sites engaged in malware distribution
# exp - Sites engaged in the housing, development or distribution of exploits
# fsa - Sites engaged in the selling or distribution of bogus or fraudulent applications 
# grm - Sites engaged in astroturfing 
# hfs - Special classification for persons caught spamming the hpHosts forums
# hjk - Sites engaged in browser hijacking or other forms of hijacking 
# pha - Sites engaged in illegal pharmacy activities
# psh - Sites engaged in Phishing
# wrz - Sites engaged in the selling, distribution or provision of warez 

# Scan HFLIST
for i in $HFLIST
do
 echo "Processing section :" $i
 # Get Online file info
 curl -sI "https://hosts-file.net/$i.txt" | grep Last-Modified > $FT/uc.hf.$i.status
 # Run basing of diff of status file
 diff $FT/uc.hf.$i.status.o $FT/uc.hf.$i.status > /dev/null
 if [ $? -ne 0 ]
 then
  echo "Status Different . processing new hostfile for class " $i
  # Saving files status in .o
  cat $FT/uc.hf.$i.status > $FT/uc.hf.$i.status.o
  # Get file
  wget -O $FT/uc.hf.$i.1.txt https://hosts-file.net/$i.txt 
  # Remove comment lines and blank lines, remove line feeds and tabs from lines (into file 2)
  sed '/#/d' $FT/uc.hf.$i.1.txt | sed '/^$/d' | sed 's/\r$//' | sed 's/\x09//g' >  $FT/uc.hf.$i.2.txt
  # Remove 127.0.0.1 and spaces from lines (into file 1)
  sed 's/127.0.0.1//g' $FT/uc.hf.$i.2.txt | sed 's/ //g'                        >  $FT/uc.hf.$i.1.txt
  # create SQL command file
  rm -f $FT/uc.hf.$i.sql
  while read -r line
  do 
   #echo $line
   sql="INSERT INTO uc SET date = '$dtd', time = '$dtt', service = 'hf.$i', name = '$line' "
   sql=$sql"ON DUPLICATE KEY UPDATE date = '$dtd', time = '$dtt', service = 'hf.$i';"
   echo $sql >> $FT/uc.hf.$i.sql
  done < $FT/uc.hf.$i.1.txt
  # Run SQL for insert data 
  mysql -u root -p$MYSQLROOT -D mailscanner < /tmp/uc/uc.hf.$i.sql
 fi 
 # Remove unneeded files after sleep
 sleep 1
 rm -f $FT/uc.hf.$i.status
 rm -f $FT/uc.hf.$i.1.txt
 rm -f $FT/uc.hf.$i.2.txt
 rm -f $FT/uc.hf.$i.sql
done


# PHISHTANK.COM
# -------------
# This service classify phishing sites
# Preferred registration and download using ID
# http://data.phishtank.com/data/$PTID/online-valid.csv

# Get Online file info
curl -sI "http://data.phishtank.com/data/$PHID/online-valid.csv" | grep Last-Modified > $FT/uc.pt.status
# Run basing of diff of status file
diff $FT/uc.pt.status.o $FT/uc.pt.status > /dev/null
if [ $? -ne 0 ]
then
 echo "Status Different . processing" 
 # Saving files status in .o
 cat $FT/uc.pt.status > $FT/uc.pt.status.o
 # Get file
 wget -O $FT/uc.pt.1.txt http://data.phishtank.com/data/$PTID/online-valid.csv
 # Read file and create SQL 
 rm -f $FT/uc.pt.sql
 while read -r line
 do
  #echo $line
  # Extract Url that is in second field of csv
  read dummy url dummy <<<$(IFS=","; echo $line)
  #echo $url
  # Extract hostname only from url
  host=`echo $url | awk -F/ '{print $3}'`
  #echo $host
  # create SQL command file
  sql="INSERT INTO uc SET date = '$dtd', time = '$dtt', service = 'pt', name = '$host' "
  sql=$sql"ON DUPLICATE KEY UPDATE date = '$dtd', time = '$dtt', service = 'pt';"
  echo $sql >> $FT/uc.pt.sql
 done < $FT/uc.pt.1.txt
 # Run SQL for insert data 
 mysql -u root -p$MYSQLROOT -D mailscanner < /tmp/uc/uc.pt.sql
 # Remove unneeded files after sleep
 sleep 1
 rm -f $FT/uc.pt.status
 rm -f $FT/uc.pt.1.txt
 rm -f $FT/uc.pt.sql
fi



